@@ -17,6 +17,7 @@ from __future__ import print_function
 
 from __future__ import absolute_import
 import numpy as np
+from numba import jit
 
 from threedigrid.orm.models import Model
 from threedigrid.orm.fields import ArrayField
@@ -115,6 +116,7 @@ class Cells(Nodes):
 
     z_coordinate = ArrayField()
     pixel_width = ArrayField()
+    pixel_coords = BboxArrayField()
 
     def __init__(self, *args, **kwargs):
 
@@ -159,6 +161,34 @@ class Cells(Nodes):
         id = inst.filter(cell_coords__contains_point=xy).id
         return id.tolist()
 
+    def get_ids_from_pix_bbox(self, bbox, subset_name=None):
+        """
+        :param x: the x coordinate in xy_epsg_code
+        :param y: the y coordinate in xy_epsg_code
+        :param subset_name: filter on a subset of cells
+
+        :return: numpy array with cell id's for x, y
+        """
+
+        inst = self
+        if subset_name:
+            inst = self.subset(subset_name)
+        id = inst.filter(pixel_coords__in_bbox=bbox).id
+        return id.tolist()
+
+    def get_nodgrid(self, pix_bbox, subset_name=None):
+        ids = np.array(
+            self.get_ids_from_pix_bbox(pix_bbox, subset_name=subset_name)
+        )
+        return create_nodgrid(
+            self.pixel_coords[:],
+            ids[:],
+            pix_bbox[2] - pix_bbox[0],
+            pix_bbox[3] - pix_bbox[1],
+            pix_bbox[0],
+            pix_bbox[1]
+        )
+
     def __repr__(self):
         return "<orm cells instance of {}>".format(self.model_name)
 
@@ -182,6 +212,8 @@ class Grid(Model):
     nodm = ArrayField()  #
     nodn = ArrayField()
     nodk = ArrayField()
+    ip = ArrayField()
+    jp = ArrayField()
 
     def __init__(self, *args, **kwargs):
 
@@ -227,3 +259,15 @@ class Grid(Model):
 
         # flip upside-down to match geotiff
         return grid_arr[::-1, ::]
+
+
+@jit(nopython=True)
+def create_nodgrid(pixel_coords, ids, width, height, offset_i, offset_j):
+    grid_arr = np.full((height, width), -9999, dtype=np.int16)
+    for id in ids:
+        i0 = np.maximum(pixel_coords[0, id], offset_i)
+        i1 = np.minimum(pixel_coords[2, id], offset_i + width)
+        j0 = np.maximum(pixel_coords[1, id], offset_j)
+        j1 = np.minimum(pixel_coords[3, id], offset_j + height)
+        grid_arr[j0: j1, i0: i1] = id
+    return grid_arr
